@@ -19,6 +19,7 @@ import (
 	"errors"
 	"fmt"
 	"regexp"
+	"slices"
 	"strings"
 	"time"
 
@@ -210,6 +211,31 @@ type (
 
 		// ResumeConsumer resumes a paused consumer.
 		ResumeConsumer(ctx context.Context, stream string, consumer string) (*ConsumerPauseResponse, error)
+
+		// CreateOrUpdatePushConsumer creates a push consumer on a given stream with
+		// given config. If consumer already exists, it will be updated (if
+		// possible). Consumer interface is returned, allowing to consume messages.
+		CreateOrUpdatePushConsumer(ctx context.Context, stream string, cfg ConsumerConfig) (PushConsumer, error)
+
+		// CreatePushConsumer creates a push consumer on a given stream with given
+		// config. If consumer already exists and the provided configuration
+		// differs from its configuration, ErrConsumerExists is returned. If the
+		// provided configuration is the same as the existing consumer, the
+		// existing consumer is returned. Consumer interface is returned,
+		// allowing to consume messages.
+		CreatePushConsumer(ctx context.Context, stream string, cfg ConsumerConfig) (PushConsumer, error)
+
+		// UpdatePushConsumer updates an existing push consumer. If consumer does not
+		// exist, ErrConsumerDoesNotExist is returned. Consumer interface is
+		// returned, allowing to consume messages.
+		UpdatePushConsumer(ctx context.Context, stream string, cfg ConsumerConfig) (PushConsumer, error)
+
+		// PushConsumer returns an interface to an existing push consumer, allowing processing
+		// of messages. If consumer does not exist, ErrConsumerNotFound is
+		// returned.
+		//
+		// It returns ErrNotPushConsumer if the consumer is not a push consumer (deliver subject is not set).
+		PushConsumer(ctx context.Context, stream string, consumer string) (PushConsumer, error)
 	}
 
 	// StreamListOpt is a functional option for [StreamManager.ListStreams] and
@@ -261,11 +287,17 @@ type (
 
 	// APIStats reports on API calls to JetStream for this account.
 	APIStats struct {
+		// Level is the API level for this account.
+		Level int `json:"level"`
+
 		// Total is the total number of API calls.
 		Total uint64 `json:"total"`
 
 		// Errors is the total number of API errors.
 		Errors uint64 `json:"errors"`
+
+		// Inflight is the number of API calls currently in flight.
+		Inflight uint64 `json:"inflight,omitempty"`
 	}
 
 	// AccountLimits includes the JetStream limits of the current account.
@@ -309,6 +341,9 @@ type (
 		// a deadline set.
 		DefaultTimeout time.Duration
 
+		// ClientTrace enables request/response API calls tracing.
+		ClientTrace *ClientTrace
+
 		publisherOpts asyncPublisherOpts
 
 		// this is the actual prefix used in the API requests
@@ -316,7 +351,6 @@ type (
 		apiPrefix      string
 		replyPrefix    string
 		replyPrefixLen int
-		clientTrace    *ClientTrace
 	}
 
 	// ClientTrace can be used to trace API interactions for [JetStream].
@@ -522,7 +556,13 @@ func (js *jetStream) Conn() *nats.Conn {
 }
 
 func (js *jetStream) Options() JetStreamOptions {
-	return js.opts
+	opts := js.opts
+	// Return a copy of ClientTrace to prevent modification
+	if opts.ClientTrace != nil {
+		clientTraceCopy := *opts.ClientTrace
+		opts.ClientTrace = &clientTraceCopy
+	}
+	return opts
 }
 
 // CreateStream creates a new stream with given config and returns an
@@ -536,27 +576,10 @@ func (js *jetStream) CreateStream(ctx context.Context, cfg StreamConfig) (Stream
 	if cancel != nil {
 		defer cancel()
 	}
-	ncfg := cfg
-	// If we have a mirror and an external domain, convert to ext.APIPrefix.
-	if ncfg.Mirror != nil && ncfg.Mirror.Domain != "" {
-		// Copy so we do not change the caller's version.
-		ncfg.Mirror = ncfg.Mirror.copy()
-		if err := ncfg.Mirror.convertDomain(); err != nil {
-			return nil, err
-		}
-	}
 
-	// Check sources for the same.
-	if len(ncfg.Sources) > 0 {
-		ncfg.Sources = append([]*StreamSource(nil), ncfg.Sources...)
-		for i, ss := range ncfg.Sources {
-			if ss.Domain != "" {
-				ncfg.Sources[i] = ss.copy()
-				if err := ncfg.Sources[i].convertDomain(); err != nil {
-					return nil, err
-				}
-			}
-		}
+	ncfg, err := convertStreamConfigDomains(cfg)
+	if err != nil {
+		return nil, err
 	}
 
 	req, err := json.Marshal(ncfg)
@@ -586,10 +609,18 @@ func (js *jetStream) CreateStream(ctx context.Context, cfg StreamConfig) (Stream
 		if len(cfg.Sources) != len(resp.Config.Sources) {
 			return nil, ErrStreamSourceNotSupported
 		}
-		for i := range cfg.Sources {
-			if len(cfg.Sources[i].SubjectTransforms) != 0 && len(resp.Sources[i].SubjectTransforms) == 0 {
-				return nil, ErrStreamSourceMultipleFilterSubjectsNotSupported
-			}
+
+		// the sources list in the response is not ordered
+		cfgNumTransforms := make([]int, len(cfg.Sources))
+		respNumTransforms := make([]int, len(resp.Config.Sources))
+		for i, cfgSource := range cfg.Sources {
+			cfgNumTransforms[i] = len(cfgSource.SubjectTransforms)
+			respNumTransforms[i] = len(resp.Config.Sources[i].SubjectTransforms)
+		}
+		slices.Sort(cfgNumTransforms)
+		slices.Sort(respNumTransforms)
+		if !slices.Equal(cfgNumTransforms, respNumTransforms) {
+			return nil, ErrStreamSubjectTransformNotSupported
 		}
 	}
 
@@ -628,6 +659,36 @@ func (ss *StreamSource) copy() *StreamSource {
 	return &nss
 }
 
+// convertStreamConfigDomains converts domain configurations to external configurations
+// in both mirror and sources of a StreamConfig. It creates a copy of the config to avoid
+// modifying the caller's version.
+func convertStreamConfigDomains(cfg StreamConfig) (StreamConfig, error) {
+	ncfg := cfg
+	// If we have a mirror and an external domain, convert to ext.APIPrefix.
+	if ncfg.Mirror != nil && ncfg.Mirror.Domain != "" {
+		// Copy so we do not change the caller's version.
+		ncfg.Mirror = ncfg.Mirror.copy()
+		if err := ncfg.Mirror.convertDomain(); err != nil {
+			return StreamConfig{}, err
+		}
+	}
+
+	// Check sources for the same.
+	if len(ncfg.Sources) > 0 {
+		ncfg.Sources = append([]*StreamSource(nil), ncfg.Sources...)
+		for i, ss := range ncfg.Sources {
+			if ss.Domain != "" {
+				ncfg.Sources[i] = ss.copy()
+				if err := ncfg.Sources[i].convertDomain(); err != nil {
+					return StreamConfig{}, err
+				}
+			}
+		}
+	}
+
+	return ncfg, nil
+}
+
 // UpdateStream updates an existing stream. If stream does not exist,
 // ErrStreamNotFound is returned.
 func (js *jetStream) UpdateStream(ctx context.Context, cfg StreamConfig) (Stream, error) {
@@ -639,7 +700,12 @@ func (js *jetStream) UpdateStream(ctx context.Context, cfg StreamConfig) (Stream
 		defer cancel()
 	}
 
-	req, err := json.Marshal(cfg)
+	ncfg, err := convertStreamConfigDomains(cfg)
+	if err != nil {
+		return nil, err
+	}
+
+	req, err := json.Marshal(ncfg)
 	if err != nil {
 		return nil, err
 	}
@@ -666,10 +732,18 @@ func (js *jetStream) UpdateStream(ctx context.Context, cfg StreamConfig) (Stream
 		if len(cfg.Sources) != len(resp.Config.Sources) {
 			return nil, ErrStreamSourceNotSupported
 		}
-		for i := range cfg.Sources {
-			if len(cfg.Sources[i].SubjectTransforms) != 0 && len(resp.Sources[i].SubjectTransforms) == 0 {
-				return nil, ErrStreamSourceMultipleFilterSubjectsNotSupported
-			}
+
+		// the sources list in the response is not ordered
+		cfgNumTransforms := make([]int, len(cfg.Sources))
+		respNumTransforms := make([]int, len(resp.Config.Sources))
+		for i, cfgSource := range cfg.Sources {
+			cfgNumTransforms[i] = len(cfgSource.SubjectTransforms)
+			respNumTransforms[i] = len(resp.Config.Sources[i].SubjectTransforms)
+		}
+		slices.Sort(cfgNumTransforms)
+		slices.Sort(respNumTransforms)
+		if !slices.Equal(cfgNumTransforms, respNumTransforms) {
+			return nil, ErrStreamSubjectTransformNotSupported
 		}
 	}
 
@@ -756,7 +830,7 @@ func (js *jetStream) CreateOrUpdateConsumer(ctx context.Context, stream string,
 	if err := validateStreamName(stream); err != nil {
 		return nil, err
 	}
-	return upsertConsumer(ctx, js, stream, cfg, consumerActionCreateOrUpdate)
+	return upsertPullConsumer(ctx, js, stream, cfg, consumerActionCreateOrUpdate)
 }
 
 // CreateConsumer creates a consumer on a given stream with given
@@ -769,7 +843,7 @@ func (js *jetStream) CreateConsumer(ctx context.Context, stream string, cfg Cons
 	if err := validateStreamName(stream); err != nil {
 		return nil, err
 	}
-	return upsertConsumer(ctx, js, stream, cfg, consumerActionCreate)
+	return upsertPullConsumer(ctx, js, stream, cfg, consumerActionCreate)
 }
 
 // UpdateConsumer updates an existing consumer. If consumer does not
@@ -779,7 +853,7 @@ func (js *jetStream) UpdateConsumer(ctx context.Context, stream string, cfg Cons
 	if err := validateStreamName(stream); err != nil {
 		return nil, err
 	}
-	return upsertConsumer(ctx, js, stream, cfg, consumerActionUpdate)
+	return upsertPullConsumer(ctx, js, stream, cfg, consumerActionUpdate)
 }
 
 // OrderedConsumer returns an OrderedConsumer instance. OrderedConsumer
@@ -790,11 +864,15 @@ func (js *jetStream) OrderedConsumer(ctx context.Context, stream string, cfg Ord
 	if err := validateStreamName(stream); err != nil {
 		return nil, err
 	}
+	namePrefix := cfg.NamePrefix
+	if namePrefix == "" {
+		namePrefix = nuid.Next()
+	}
 	oc := &orderedConsumer{
 		js:         js,
 		cfg:        &cfg,
 		stream:     stream,
-		namePrefix: nuid.Next(),
+		namePrefix: namePrefix,
 		doReset:    make(chan struct{}, 1),
 	}
 	consCfg := oc.getConsumerConfig()
@@ -826,6 +904,49 @@ func (js *jetStream) DeleteConsumer(ctx context.Context, stream string, name str
 	return deleteConsumer(ctx, js, stream, name)
 }
 
+// CreateOrUpdatePushConsumer creates a push consumer on a given stream with
+// given config. If consumer already exists, it will be updated (if
+// possible). Consumer interface is returned, allowing to consume messages.
+func (js *jetStream) CreateOrUpdatePushConsumer(ctx context.Context, stream string, cfg ConsumerConfig) (PushConsumer, error) {
+	if err := validateStreamName(stream); err != nil {
+		return nil, err
+	}
+	return upsertPushConsumer(ctx, js, stream, cfg, consumerActionCreateOrUpdate)
+}
+
+// CreatePushConsumer creates a push consumer on a given stream with given
+// config. If consumer already exists and the provided configuration
+// differs from its configuration, ErrConsumerExists is returned. If the
+// provided configuration is the same as the existing consumer, the
+// existing consumer is returned. Consumer interface is returned,
+// allowing to consume messages.
+func (js *jetStream) CreatePushConsumer(ctx context.Context, stream string, cfg ConsumerConfig) (PushConsumer, error) {
+	if err := validateStreamName(stream); err != nil {
+		return nil, err
+	}
+	return upsertPushConsumer(ctx, js, stream, cfg, consumerActionCreate)
+}
+
+// UpdatePushConsumer updates an existing push consumer. If consumer does not
+// exist, ErrConsumerDoesNotExist is returned. Consumer interface is
+// returned, allowing to consume messages.
+func (js *jetStream) UpdatePushConsumer(ctx context.Context, stream string, cfg ConsumerConfig) (PushConsumer, error) {
+	if err := validateStreamName(stream); err != nil {
+		return nil, err
+	}
+	return upsertPushConsumer(ctx, js, stream, cfg, consumerActionUpdate)
+}
+
+// PushConsumer returns an interface to an existing consumer, allowing processing
+// of messages. If consumer does not exist, ErrConsumerNotFound is
+// returned.
+func (js *jetStream) PushConsumer(ctx context.Context, stream string, name string) (PushConsumer, error) {
+	if err := validateStreamName(stream); err != nil {
+		return nil, err
+	}
+	return getPushConsumer(ctx, js, stream, name)
+}
+
 func (js *jetStream) PauseConsumer(ctx context.Context, stream string, consumer string, pauseUntil time.Time) (*ConsumerPauseResponse, error) {
 	if err := validateStreamName(stream); err != nil {
 		return nil, err
