@@ -58,6 +58,9 @@ type (
 
 		// Subject returns underlying NATS message subject.
 		Subject() string
+
+		// Reply returns underlying NATS message reply subject.
+		Reply() string
 	}
 
 	// Headers is a wrapper around [*nats.Header]
@@ -71,6 +74,11 @@ type (
 		msg          *nats.Msg
 		respondError error
 	}
+
+	serviceError struct {
+		Code        string `json:"code"`
+		Description string `json:"description"`
+	}
 )
 
 var (
@@ -144,6 +152,11 @@ func (r *request) Error(code, description string, data []byte, opts ...RespondOp
 		r.respondError = err
 		return err
 	}
+	r.respondError = &serviceError{
+		Code:        code,
+		Description: description,
+	}
+
 	return nil
 }
 
@@ -176,6 +189,11 @@ func (r *request) Subject() string {
 	return r.msg.Subject
 }
 
+// Reply returns underlying NATS message reply subject.
+func (r *request) Reply() string {
+	return r.msg.Reply
+}
+
 // Get gets the first value associated with the given key.
 // It is case-sensitive.
 func (h Headers) Get(key string) string {
@@ -187,3 +205,7 @@ func (h Headers) Get(key string) string {
 func (h Headers) Values(key string) []string {
 	return nats.Header(h).Values(key)
 }
+
+func (e *serviceError) Error() string {
+	return fmt.Sprintf("%s:%s", e.Code, e.Description)
+}
