@@ -483,6 +483,54 @@ func TestKeyValueWatch(t *testing.T) {
 			t.Fatalf("Stop watcher did not return")
 		}
 	})
+
+	// Test channel-based error API integration with select patterns
+	t.Run("error channel with select", func(t *testing.T) {
+		s := RunBasicJetStreamServer()
+		defer shutdownJSServerAndRemoveStorage(t, s)
+
+		nc, js := jsClient(t, s)
+		defer nc.Close()
+
+		kv, err := js.CreateKeyValue(&nats.KeyValueConfig{Bucket: "WATCH_ERROR"})
+		expectOk(t, err)
+
+		// Put some initial keys
+		_, err = kv.Put("test1", []byte("value1"))
+		expectOk(t, err)
+		_, err = kv.Put("test2", []byte("value2"))
+		expectOk(t, err)
+
+		watcher, err := kv.WatchAll()
+		expectOk(t, err)
+		defer watcher.Stop()
+
+		updateCount := 0
+		var watchCompleted bool
+
+	Outer:
+		for !watchCompleted {
+			select {
+			case entry := <-watcher.Updates():
+				if entry == nil {
+					break Outer
+				}
+				updateCount++
+
+			case err := <-watcher.Error():
+				if err != nil {
+					t.Fatalf("Unexpected error from watcher error channel: %v", err)
+				}
+
+			case <-time.After(2 * time.Second):
+				t.Fatalf("Timeout waiting for watcher completion")
+			}
+		}
+
+		if updateCount < 2 {
+			t.Fatalf("Expected at least 2 updates, got %d", updateCount)
+		}
+	})
 }
 
 func TestKeyValueWatchContext(t *testing.T) {
@@ -885,6 +933,56 @@ func TestKeyValueListKeys(t *testing.T) {
 	if _, ok := kmap["age"]; !ok {
 		t.Fatalf("Expected %q to be only key present", "age")
 	}
+
+	// Test channel-based error API patterns
+	t.Run("error channel with select", func(t *testing.T) {
+		keys, err := kv.ListKeys()
+		expectOk(t, err)
+		defer keys.Stop()
+
+		var keyList []string
+		var completed bool
+
+		for !completed {
+			select {
+			case key, ok := <-keys.Keys():
+				if !ok {
+					completed = true
+					break
+				}
+				keyList = append(keyList, key)
+
+			case err := <-keys.Error():
+				if err != nil {
+					t.Fatalf("Unexpected error from error channel: %v", err)
+				}
+			}
+		}
+
+		if len(keyList) != 1 {
+			t.Fatalf("Expected 1 key using select pattern, got %d", len(keyList))
+		}
+	})
+
+	t.Run("error check after completion", func(t *testing.T) {
+		keys, err := kv.ListKeys()
+		expectOk(t, err)
+		defer keys.Stop()
+
+		var keyList []string
+		for key := range keys.Keys() {
+			keyList = append(keyList, key)
+		}
+
+		// Check for errors after completion - should not block and return nil
+		if err := <-keys.Error(); err != nil {
+			t.Fatalf("Unexpected error after completion: %v", err)
+		}
+
+		if len(keyList) != 1 {
+			t.Fatalf("Expected 1 key after completion check, got %d", len(keyList))
+		}
+	})
 }
 
 func TestKeyValueCrossAccounts(t *testing.T) {
@@ -1390,7 +1488,7 @@ func TestKeyValueNonDirectGet(t *testing.T) {
 		t.Fatalf("Error on get: v=%+v err=%v", v, err)
 	}
 	if v, err := kvi.GetRevision("key1", 1); err != nil || string(v.Value()) != "val1" {
-		t.Fatalf("Error on get revisiong: v=%+v err=%v", v, err)
+		t.Fatalf("Error on get revision: v=%+v err=%v", v, err)
 	}
 	if v, err := kvi.GetRevision("key1", 2); err == nil {
 		t.Fatalf("Expected error, got %+v", v)
@@ -1755,3 +1853,84 @@ func TestKeyValueWatcherStopTimer(t *testing.T) {
 	}
 	time.Sleep(500 * time.Millisecond)
 }
+
+func TestKeyValueListKeysDuplicates(t *testing.T) {
+	listKeysF := func(kv nats.KeyValue) ([]string, error) {
+		t.Helper()
+		lister, err := kv.ListKeys()
+		if err != nil {
+			return nil, fmt.Errorf("error listing keys: %v", err)
+		}
+		var keys []string
+		for key := range lister.Keys() {
+			keys = append(keys, key)
+		}
+		return keys, nil
+	}
+
+	keysF := func(kv nats.KeyValue) ([]string, error) {
+		t.Helper()
+		return kv.Keys()
+	}
+
+	for _, test := range []string{"ListKeys", "Keys"} {
+		t.Run(test, func(t *testing.T) {
+			s := RunBasicJetStreamServer()
+			defer shutdownJSServerAndRemoveStorage(t, s)
+
+			nc, js := jsClient(t, s)
+			defer nc.Close()
+
+			kv, err := js.CreateKeyValue(&nats.KeyValueConfig{Bucket: "TEST_KV", History: 5})
+			if err != nil {
+				t.Fatalf("Error creating KV: %v", err)
+			}
+
+			for i := range 10 {
+				key := fmt.Sprintf("key_%d", i)
+				if _, err := kv.PutString(key, "initial"); err != nil {
+					t.Fatalf("Error putting key %s: %v", key, err)
+				}
+			}
+
+			done := make(chan bool)
+			go func() {
+				// Continuously update existing keys
+				for {
+					select {
+					case <-done:
+						return
+					default:
+						for i := range 5 {
+							key := fmt.Sprintf("key_%d", i)
+							kv.PutString(key, "updated")
+						}
+					}
+				}
+			}()
+
+			// List keys multiple times while updates are happening
+			for range 20 {
+				var keys []string
+				if test == "Keys" {
+					keys, err = keysF(kv)
+				} else {
+					keys, err = listKeysF(kv)
+				}
+				if err != nil {
+					t.Fatalf("Error getting keys: %v", err)
+				}
+
+				seen := make(map[string]struct{})
+				for _, key := range keys {
+					if _, exists := seen[key]; exists {
+						t.Fatalf("Duplicate key found: %s", key)
+					}
+					seen[key] = struct{}{}
+				}
+			}
+
+			close(done)
+		})
+	}
+}
